ENCODING = 'gb2312'
GATEWAY_NAME = 'cats_uni'


class FILES:
    TASKS = 'instructions'  #下单
    CANCEL = 'instructions'
    ORDERS = 'order_updates'
    ALGO_ORDERS = 'stats_all'
    TRADES = 'trade'
    POSITIONS = 'asset'